package de.ozgcloud.vorgang.servicekonto;

import java.util.List;
import java.util.Set;

import jakarta.validation.constraints.NotEmpty;

import org.apache.commons.lang3.StringUtils;
import org.springframework.validation.annotation.Validated;

import lombok.Builder;
import lombok.Getter;
import lombok.Singular;
import lombok.extern.log4j.Log4j2;

@Validated
@Getter
@Builder
@Log4j2
public class ServiceKonto {

	public static final String OSI_IDENTIFIER = "OSI";
	private static final Set<String> EXPECTED_TRUST_LEVELS = Set.of("STORK-QAA-Level-1", "STORK-QAA-Level-2", "STORK-QAA-Level-3",
			"STORK-QAA-Level-4");

	private String type;
	private String trustLevel;

	@NotEmpty
	@Singular
	private List<PostfachAddress> postfachAddresses;

	public static class ServiceKontoBuilder {

		public ServiceKontoBuilder trustLevel(String trustLevel) {
			validateTrustLevel(trustLevel);
			this.trustLevel = trustLevel;
			return this;
		}

		void validateTrustLevel(String trustLevel) {
			if (StringUtils.isNotEmpty(trustLevel) && !EXPECTED_TRUST_LEVELS.contains(trustLevel)) {
				logError(trustLevel);
			}
		}

		void logError(String trustLevelValue) {
			LOG.error("Unexpected trust level: {}", trustLevelValue);
		}
	}
}