/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.List;

import org.springframework.data.annotation.Id;
import org.springframework.data.annotation.TypeAlias;
import org.springframework.data.mongodb.core.mapping.Document;

import de.ozgcloud.vorgang.clientattribute.ClientAttributesMap;
import de.ozgcloud.vorgang.vorgang.redirect.Forwarding;
import lombok.Builder;
import lombok.Getter;
import lombok.Singular;

@Getter
@Builder(toBuilder = true)
@Document(collection = Vorgang.COLLECTION_NAME)
@TypeAlias("Vorgang")
public class Vorgang {
	public static final String COLLECTION_NAME = "vorgang";

	public static final String MONGODB_FIELDNAME_ID = "_id";
	public static final String MONGODB_FIELDNAME_VERSION = "version";
	public static final String MONGODB_FIELDNAME_STATUS = "status";
	static final String MONGODB_FIELDNAME_CREATED_AT = "createdAt";
	public static final String MONGODB_FIELDNAME_ASSIGNED_TO = "assignedTo";
	public static final String MONGODB_FIELDNAME_IN_CREATION = "inCreation";
	public static final String MONGODB_FIELDNAME_HEADER = "header";

	public static final String FIELD_EINGANGSKENNZ = "eingangs.header.requestId";
	public static final String FIELD_AKTENZEICHEN = "aktenzeichen";
	public static final String FIELD_ANTRAGSTELLER_VORNAME = "eingangs.antragsteller.vorname";
	public static final String FIELD_ANTRAGSTELLER_NACHNAME = "eingangs.antragsteller.nachname";
	public static final String FIELD_NAME = "name";
	public static final String FIELD_NUMMER = "nummer";

	public static final String FIELD_ORGANISATIONSEINHEIT = "eingangs.zustaendigeStelle.organisationseinheitenId";
	public static final String FIELD_CLIENT_ATTRIBUTES = "clientAttributes";

	public enum Status {
		NEU, ANGENOMMEN, VERWORFEN, IN_BEARBEITUNG, BESCHIEDEN, ABGESCHLOSSEN, WEITERGELEITET, ZU_LOESCHEN, DELETED;
	}

	@Id
	private String id;

	private long version;
	private boolean inCreation;

	private String name;
	private String nummer;
	private String formEngineName;

	@Builder.Default
	private ZonedDateTime createdAt = ZonedDateTime.now(ZoneId.of("UTC"));
	private String assignedTo;

	@Builder.Default
	private Status status = Status.NEU;
	private String aktenzeichen;

	private VorgangHead header;

	@Singular
	private List<Eingang> eingangs;

	@Singular
	private List<Forwarding> forwardings;

	@Builder.Default
	private ClientAttributesMap clientAttributes = new ClientAttributesMap();

}