/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.NullValuePropertyMappingStrategy;
import org.springframework.beans.factory.annotation.Autowired;

import de.ozgcloud.vorgang.clientattribute.ClientAttributeMapper;
import de.ozgcloud.vorgang.servicekonto.ServiceKontoMapper;

@Mapper(nullValuePropertyMappingStrategy = NullValuePropertyMappingStrategy.IGNORE, //
		nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS, //
		uses = { EingangMapper.class, ClientAttributeMapper.class, ServiceKontoMapper.class })
public abstract class VorgangWithEingangMapper {

	@Autowired
	public ClientAttributeMapper clientAttributeMapper;

	GrpcVorgangWithEingang toVorgangWithEingang(Vorgang vorgang) {
		var mapped = toGrpcWithoutClientAttributes(vorgang);
		return mapped.toBuilder()
				.addAllClientAttributes(clientAttributeMapper.mapClientAttributes(vorgang.getClientAttributes()))
				.build();
	}

	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "statusBytes", ignore = true)
	@Mapping(target = "removeClientAttributes", ignore = true)
	@Mapping(target = "nummerBytes", ignore = true)
	@Mapping(target = "nameBytes", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "mergeHeader", ignore = true)
	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "mergeEingang", ignore = true)
	@Mapping(target = "idBytes", ignore = true)
	@Mapping(target = "headerOrBuilder", ignore = true)
	@Mapping(target = "formEngineNameBytes", ignore = true)
	@Mapping(target = "eingangOrBuilder", ignore = true)
	@Mapping(target = "defaultInstanceForType", ignore = true)
	@Mapping(target = "createdAtBytes", ignore = true)
	@Mapping(target = "clientAttributesOrBuilderList", ignore = true)
	@Mapping(target = "clientAttributesList", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	@Mapping(target = "assignedToBytes", ignore = true)
	@Mapping(target = "allFields", ignore = true)
	@Mapping(target = "aktenzeichenBytes", ignore = true)
	@Mapping(target = "clientAttributesBuilderList", ignore = true)
	@Mapping(target = "eingang", source = "eingangs")
	abstract GrpcVorgangWithEingang toGrpcWithoutClientAttributes(Vorgang vorgang);

	Eingang mapListToGrpcEingang(List<Eingang> eingaenge) {
		if (CollectionUtils.isEmpty(eingaenge)) {
			return null;
		}
		return eingaenge.get(0);
	}
}