/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.clientattribute;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.context.ActiveProfiles;

import de.ozgcloud.vorgang.callcontext.TestCallContextAttachingInterceptor;
import de.ozgcloud.vorgang.common.security.PolicyService;
import de.ozgcloud.vorgang.grpc.clientAttribute.ClientAttributeServiceGrpc.ClientAttributeServiceBlockingStub;
import net.devh.boot.grpc.client.inject.GrpcClient;

@SpringBootTest(properties = {
		"grpc.server.inProcessName=test",
		"grpc.server.port=-1",
		"grpc.client.inProcess.address=in-process:test"
})
@DirtiesContext
@ActiveProfiles("itcase")
class GrpcClientAttributeServiceITCase {

	@GrpcClient("inProcess")
	private ClientAttributeServiceBlockingStub serviceStub;

	@MockBean
	private ClientAttributeService service;
	@MockBean
	private PolicyService policyService;

	@BeforeEach
	void mockPolicyService() {
		when(policyService.hasOrganisationEinheitenId(anyString())).thenReturn(true);
	}

	@Nested
	class TestSetAttribute {

		@Captor
		private ArgumentCaptor<ClientAttribute> clientAttributeCaptor;

		@Test
		void shouldCallService() {
			prepareServiceStub().set(GrpcSetClientAttributeRequestTestFactory.create());

			verify(service).set(any(), any());
		}

		@Test
		void shouldFillAttribut() {
			prepareServiceStub().set(GrpcSetClientAttributeRequestTestFactory.create());

			verify(service).set(any(), clientAttributeCaptor.capture());
			assertThat(clientAttributeCaptor.getValue()).usingRecursiveComparison().isEqualTo(ClientAttributeTestFactory.create());
		}

	}

	private ClientAttributeServiceBlockingStub prepareServiceStub() {
		return serviceStub.withInterceptors(new TestCallContextAttachingInterceptor());
	}
}
