package de.ozgcloud.vorgang.common.userconfig;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.user.grpc.organisationseinheit.OrganisationsEinheitServiceGrpc.OrganisationsEinheitServiceBlockingStub;
import de.ozgcloud.vorgang.callcontext.VorgangManagerClientCallContextAttachingInterceptor;

class UserConfigRemoteServiceTest {
	@Spy
	@InjectMocks
	private UserConfigRemoteService remoteService;

	@Mock
	private OrganisationsEinheitServiceBlockingStub stub;

	@DisplayName("Test getting Organsiationseinheiten Ids of the users")
	@Nested
	class TestGettingOrganisationsEinheitenIds {
		@BeforeEach
		void init() {
			when(stub.getSupportedOrganisationsEinheiten(any()))
					.thenReturn(GrpcGetSupportedOrganisationEinheitenResponseTestFactory.create());
			when(stub.withInterceptors(any())).thenReturn(stub);
		}

		@Test
		void shouldGetOrganisationsEinheiten() {
			var organisationsEinheiten = remoteService.getSupportedOrganisationsEinheiten();

			assertThat(organisationsEinheiten).hasSize(1)
					.contains(GrpcGetSupportedOrganisationEinheitenResponseTestFactory.ORGANISATIONS_EINHEIT_ID);
		}

		@Test
		void shouldUseInterceptor() {
			remoteService.getSupportedOrganisationsEinheiten();

			verify(stub).withInterceptors(any(VorgangManagerClientCallContextAttachingInterceptor.class));
		}
	}
}
